#include <stdio.h>
#include "TROOT.h"
#include "TApplication.h"
#include "CTDResol.h"

//_____________________________________________________________________
//  -------------------------------
//  void CTDResol(p,tanl,sgk,sgkip)
//  -------------------------------
//  (Inputs)
//	Double_t p	:	momentum (GeV)
//	Double_t tanl	:	tan(lambda)
//  (Outputs)
//	Double_t &sgk	:	sigma_kappa w/o IP constraint (1/GeV)
//	Double_t &sgkip :	sigma_kappa w/  IP constraint (1/GeV)
//

void CTDResol(Double_t p, Double_t tanl, Double_t &sgk, Double_t &sgkip)
{
   // ------------------------------
   // Initialize material properties
   // ------------------------------

   Double_t x0vac   = 0.;
   Double_t x0be    = CTDLayer::kX0Be;
   Double_t x0air   = CTDLayer::kX0Air;
   Double_t x0si    = CTDLayer::kX0Si;
   Double_t x0cfrp  = CTDLayer::kX0CFRP;
   Double_t x0cdc   = CTDLayer::kX0CDC;

   // ------------------------------
   // Initialize detector properties
   // ------------------------------

   Double_t sgxdum  = 1.e1;	// dummy sigma_x (cm)
   Double_t bfield  = 3.;	// B field (T)

   Int_t    withvtx = 1;	// (0,1) = (w/o,w/) VTX
   Double_t sgxvtx  = 0.0004;	// sigma_x(VTX) (cm)

   Int_t    withimt = 1;	// (0,1) = (w/o,w/) IMT
   Double_t sgximt  = 0.0040;	// sigma_x(IMT) (cm)

   Int_t    nscdc   = 50;	// N_samples of CDC
   Double_t rincdc  = 45.;	// R_in(CDC)    (cm)
   Double_t routcdc = 155.;	// R_out(CDC)   (cm)
   Double_t sgxcdc  = 0.0085;	// sigma_x(CDC) (cm)
   Int_t    ns;
   Double_t elcdc;
   if (tanl <= 1.) {
      elcdc = routcdc - rincdc;
      ns    = nscdc;
   } else {
      elcdc = routcdc/tanl - rincdc;
      ns    = TMath::Nint(nscdc*elcdc/(routcdc-rincdc));
   }
   if (ns < 3) {
      cerr << "Not enough sampling points: ns = " << ns << endl;
   }

   // ------------------------------------
   // Initialize the tracker configuration
   // ------------------------------------
   // [1] How to define a layer
   //     Create a CTDLayer object with
   // 	    CTDLayer(name,Nsample,Rin,thickness,Bfield,X0,sgimax)
   //     wehere
   //		Char_t  *name		: name of the layer
   //		Int_t    Nsample	: No. samples in this layer
   //		Double_t Rin		: inner radius (cm)
   //		Double_t thickness	: thickness of the layer (cm)
   //		Double_t Bfield		: B field (T)
   //		Double_t X0		: radiation length (1/cm)
   //		Double_t sigmax		: resolution per sample (cm)
   //     Alternatively, one can use
   //	    CTDLayer(layer,Rin)
   //     where
   //		const CTDLayer &layer	: previous layer of the same kind
   //		Rin			: inner radius of the new layer (cm)
   //	  if some layer that only differs in Rin has already been defined.
   //	  The defined layer should be added to a CTDTracker object.
   //	  Once layers other than air are added, one can fill the
   //	  inter-layer dead gaps by air by calling
   //	    FillGapByAir(x0air)
   //	  member function of the CTDTracker class.
   //

   CTDTracker tracker;

   CTDLayer vac("Vacuum",0,0.0,2.00,bfield,x0vac,sgxdum); tracker.Add(&vac);
   CTDLayer bp("BM Pipe",0,2.0,0.05,bfield,x0be ,sgxdum); tracker.Add(&bp);
   CTDLayer vtx1("VTX",withvtx,2.4,0.03,bfield,x0si,sgxvtx); tracker.Add(&vtx1);
   CTDLayer vtx2(vtx1,3.6); tracker.Add(&vtx2);
   CTDLayer vtx3(vtx1,4.8); tracker.Add(&vtx3);
   CTDLayer vtx4(vtx1,6.0); tracker.Add(&vtx4);
   CTDLayer imt1("IMT",withimt,10.,0.03,bfield,x0si,sgximt); tracker.Add(&imt1);
   CTDLayer imt2(imt1,17.); tracker.Add(&imt2);
   CTDLayer imt3(imt1,24.); tracker.Add(&imt3);
   CTDLayer imt4(imt1,31.); tracker.Add(&imt4);
   CTDLayer imt5(imt1,38.); tracker.Add(&imt5);
   CTDLayer st("SUP",0,40.0,0.5,bfield,x0cfrp,sgxdum); tracker.Add(&st);
   CTDLayer cdc1("CDC",0,40.5,4.5,bfield,x0cdc,sgxdum); tracker.Add(&cdc1);
   CTDLayer cdc2("CDC",ns,45.0,elcdc,bfield,x0cdc,sgxcdc); tracker.Add(&cdc2);

   tracker.FillGapByAir(x0air);		// fills gaps by air

   // ------------------------------------
   // Print out the tracker configuration
   // ------------------------------------

   tracker.DebugPrint();

   // ------------------------------------
   // Calculate momentum resolution
   // ------------------------------------

   sgk   = tracker.GetSigmaKappa(p,tanl);	// sigma_kappa w/o IP
   sgkip = tracker.GetSigmaKappaIP(p,tanl);	// sigma_kappa w/  IP
}


//_____________________________________________________________________

TROOT root("CTDResol","Charged Track Detector Resolution Study");

int main(int argc, char **argv)
{
   // ---------------
   // Initialize ROOT
   // ---------------

   TApplication theApp("App", &argc, argv);

   // ---------------
   // Parse Arguments
   // ---------------

   Double_t p, costh;
   switch (argc) {
      case 2:
         sscanf(argv[1],"%lg",&p);
         costh = 0;
         break;
      case 3:
         sscanf(argv[1],"%lg",&p);
         sscanf(argv[2],"%lg",&costh);
         break;
      default:
         p     = 50.;
         costh = 0;
         break;
   }
   Double_t tanl = costh/TMath::Sqrt((1-costh)*(1+costh));

   Double_t sgk, sgkip;
   CTDResol(p,tanl,sgk,sgkip);

   // ------------------------------------
   // Calculate momentum resolution
   // ------------------------------------

   cout << "p = " << p << " (GeV)"
        << " cos(theta) = " << costh << endl;   
   cout << "sigma_kappa w/o ip = " << sgk   << " (1/GeV)" << endl
        << "            w/  ip = " << sgkip << " (1/GeV)" << endl;

   // theApp.Run();

   return 0;
}
